"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildLiquidateIxs = void 0;
const liquidateObligationAndRedeemReserveCollateral_1 = require("../instructions/liquidateObligationAndRedeemReserveCollateral.js");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const __1 = require("../index.js");
const constants_1 = require("../core/constants.js");
const PRICE_PAIRS = (0, constants_1.getPricePair)();
const REUSD_PRICE = (0, constants_1.getReUSDPricePair)();
const REVND_PRICE = constants_1.PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[0];
const RENGN_PRICE = constants_1.PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[1];
const buildLiquidateIxs = async (connection, obligation, liquidityAmount, depositReserve, borrowReserve, payer) => {
    const ixs = [];
    const lendingMarket = (0, constants_1.getRelendInfo)()[0];
    const programId = (0, __1.getProgramId)();
    const obligationAccountInfo = await connection.getAccountInfo(obligation);
    let obligationDetails = null;
    const depositReserves = [];
    const borrowReserves = [];
    if (obligationAccountInfo) {
        obligationDetails = (0, __1.parseObligation)(web3_js_1.PublicKey.default, obligationAccountInfo).info;
        obligationDetails.deposits.forEach((deposit) => {
            depositReserves.push(deposit.depositReserve);
        });
        obligationDetails.borrows.forEach((borrow) => {
            borrowReserves.push(borrow.borrowReserve);
        });
    }
    const selectedDepositReserve = lendingMarket.reserves.find((r) => r.address === depositReserve.toBase58());
    const selectedBorrowReserve = lendingMarket.reserves.find((r) => r.address === borrowReserve.toBase58());
    if (!selectedDepositReserve || !selectedBorrowReserve) {
        throw new Error("Couldn't find collateral or loan reserve.");
    }
    // refresh reserve
    const uniqueAllReserveAddresses = [...new Set([selectedDepositReserve?.address, selectedBorrowReserve?.address])];
    const pda = (0, constants_1.getPDA)();
    uniqueAllReserveAddresses.forEach((reserveAddress) => {
        const reserveInfo = lendingMarket.reserves.find((reserve) => reserve.address === reserveAddress);
        if (!reserveInfo) {
            throw new Error(`Could not find asset ${reserveAddress} in reserves`);
        }
        const symbol = reserveInfo.liquidityToken.symbol;
        let pair;
        if (symbol === constants_1.RE_USD) {
            pair = REUSD_PRICE;
        }
        else if (symbol === constants_1.RE_VND) {
            pair = REVND_PRICE;
        }
        else if (symbol === constants_1.RE_NGN) {
            pair = RENGN_PRICE;
        }
        else {
            pair = PRICE_PAIRS.find((pair) => pair.baseSymbol === reserveInfo.liquidityToken.symbol);
        }
        const oracleProduct = pda.product(new web3_js_1.PublicKey(pair.quoteToken), new web3_js_1.PublicKey(pair.baseToken));
        const refreshReserveIx = (0, __1.refreshReserveInstruction)(new web3_js_1.PublicKey(reserveAddress), programId, oracleProduct.key, new web3_js_1.PublicKey(reserveInfo.pythOracle), new web3_js_1.PublicKey(reserveInfo.switchboardOracle));
        ixs.push(refreshReserveIx);
    });
    const refreshObligationIx = (0, __1.refreshObligationInstruction)(obligation, depositReserves, borrowReserves, programId);
    ixs.push(refreshObligationIx);
    // get account that will be repaying the reserve liquidity
    const repayAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedBorrowReserve.liquidityToken.mint), payer);
    const rewardedWithdrawalCollateralAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress), payer);
    const rewardedWithdrawalCollateralAccountInfo = await connection.getAccountInfo(rewardedWithdrawalCollateralAccount);
    if (!rewardedWithdrawalCollateralAccountInfo) {
        const createUserCollateralAccountIx = spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress), rewardedWithdrawalCollateralAccount, payer, payer);
        ixs.push(createUserCollateralAccountIx);
    }
    const rewardedWithdrawalLiquidityAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedDepositReserve.liquidityToken.mint), payer);
    const rewardedWithdrawalLiquidityAccountInfo = await connection.getAccountInfo(rewardedWithdrawalLiquidityAccount);
    if (!rewardedWithdrawalLiquidityAccountInfo) {
        const createUserLiquidityAccountIx = spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(selectedDepositReserve.liquidityToken.mint), rewardedWithdrawalLiquidityAccount, payer, payer);
        ixs.push(createUserLiquidityAccountIx);
    }
    ixs.push((0, liquidateObligationAndRedeemReserveCollateral_1.LiquidateObligationAndRedeemReserveCollateral)(liquidityAmount, repayAccount, rewardedWithdrawalCollateralAccount, rewardedWithdrawalLiquidityAccount, new web3_js_1.PublicKey(selectedBorrowReserve.address), new web3_js_1.PublicKey(selectedBorrowReserve.liquidityAddress), new web3_js_1.PublicKey(selectedDepositReserve.address), new web3_js_1.PublicKey(selectedDepositReserve.collateralMintAddress), new web3_js_1.PublicKey(selectedDepositReserve.collateralSupplyAddress), new web3_js_1.PublicKey(selectedDepositReserve.liquidityAddress), new web3_js_1.PublicKey(selectedDepositReserve.liquidityFeeReceiverAddress), obligation, new web3_js_1.PublicKey(lendingMarket.address), new web3_js_1.PublicKey(lendingMarket.authorityAddress), payer, programId));
    return ixs;
};
exports.buildLiquidateIxs = buildLiquidateIxs;
